@@ -8,6 +8,7 @@
  */
 
 #include <glib.h>
+#include <gudev/gudev.h>
 
 typedef struct {
 	float x;
@@ -15,6 +16,8 @@ typedef struct {
 	float z;
 } AccelVec3;
 
+AccelVec3 *setup_mount_matrix (GUdevDevice *device);
+
 gboolean parse_mount_matrix (const char *mtx,
                              AccelVec3  *vecs[3]);
 
