@@ -30,11 +30,7 @@ static DrvData *drv_data = NULL;
 static gboolean
 hwmon_light_discover (GUdevDevice *device)
 {
-	if (g_strcmp0 (g_udev_device_get_property (device, "IIO_SENSOR_PROXY_TYPE"), "hwmon-als") != 0)
-		return FALSE;
-
-	g_debug ("Found HWMon light at %s", g_udev_device_get_sysfs_path (device));
-	return TRUE;
+	return drv_check_udev_sensor_type (device, "hwmon-als", "HWMon light");
 }
 
 static gboolean
@@ -43,7 +39,7 @@ light_changed (gpointer user_data)
 	LightReadings readings;
 	gdouble level;
 	char *contents;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	if (g_file_get_contents (drv_data->light_path, &contents, NULL, &error)) {
 		int light1, light2;
@@ -57,7 +53,6 @@ light_changed (gpointer user_data)
 	} else {
 		g_warning ("Failed to read input level at %s: %s",
 			   drv_data->light_path, error->message);
-		g_error_free (error);
 		return G_SOURCE_CONTINUE;
 	}
 
