@@ -146,11 +146,7 @@ read_heading (gpointer user_data)
 static gboolean
 iio_buffer_compass_discover (GUdevDevice *device)
 {
-	if (g_strcmp0 (g_udev_device_get_property (device, "IIO_SENSOR_PROXY_TYPE"), "iio-buffer-compass") != 0)
-		return FALSE;
-
-	g_debug ("Found IIO buffer compass at %s", g_udev_device_get_sysfs_path (device));
-	return TRUE;
+	return drv_check_udev_sensor_type (device, "iio-buffer-compass", "IIO buffer compass");
 }
 
 static gboolean
