@@ -25,8 +25,7 @@ typedef struct DrvData {
 	const char         *name;
 	AccelVec3          *mount_matrix;
 	AccelLocation       location;
-
-	double              scale;
+	gdouble             scale;
 } DrvData;
 
 static DrvData *drv_data = NULL;
@@ -87,8 +86,8 @@ static gboolean
 iio_poll_accel_discover (GUdevDevice *device)
 {
 	/* We also handle devices with trigger buffers, but there's no trigger available on the system */
-	if (g_strcmp0 (g_udev_device_get_property (device, "IIO_SENSOR_PROXY_TYPE"), "iio-poll-accel") != 0 &&
-	    g_strcmp0 (g_udev_device_get_property (device, "IIO_SENSOR_PROXY_TYPE"), "iio-buffer-accel") != 0)
+	if (!drv_check_udev_sensor_type (device, "iio-poll-accel", NULL) &&
+	    !drv_check_udev_sensor_type (device, "iio-buffer-accel", NULL))
 		return FALSE;
 
 	g_debug ("Found IIO poll accelerometer at %s", g_udev_device_get_sysfs_path (device));
@@ -128,9 +127,7 @@ iio_poll_accel_open (GUdevDevice        *device,
 	drv_data->location = setup_accel_location (device);
 	drv_data->callback_func = callback_func;
 	drv_data->user_data = user_data;
-	drv_data->scale = g_udev_device_get_sysfs_attr_as_double (device, "in_accel_scale");
-	if (drv_data->scale == 0.0)
-		drv_data->scale = 1.0;
+	drv_data->scale = get_accel_scale (device);
 
 	return TRUE;
 }
