@@ -24,6 +24,8 @@ typedef struct DrvData {
 	char               *input_path;
 	guint               interval;
 	guint               timeout_id;
+
+	double              scale;
 } DrvData;
 
 static DrvData *drv_data = NULL;
@@ -31,11 +33,7 @@ static DrvData *drv_data = NULL;
 static gboolean
 iio_poll_light_discover (GUdevDevice *device)
 {
-	if (g_strcmp0 (g_udev_device_get_property (device, "IIO_SENSOR_PROXY_TYPE"), "iio-poll-als") != 0)
-		return FALSE;
-
-	g_debug ("Found IIO poll light at %s", g_udev_device_get_sysfs_path (device));
-	return TRUE;
+	return drv_check_udev_sensor_type (device, "iio-poll-als", "IIO poll als");
 }
 
 static gboolean
@@ -44,7 +42,7 @@ light_changed (gpointer user_data)
 	LightReadings readings;
 	gdouble level;
 	char *contents;
-	GError *error = NULL;
+	g_autoptr(GError) error = NULL;
 
 	if (g_file_get_contents (drv_data->input_path, &contents, NULL, &error)) {
 		level = g_ascii_strtod (contents, NULL);
@@ -52,11 +50,10 @@ light_changed (gpointer user_data)
 	} else {
 		g_warning ("Failed to read input level at %s: %s",
 			   drv_data->input_path, error->message);
-		g_error_free (error);
 		return G_SOURCE_CONTINUE;
 	}
-
-	readings.level = level;
+	readings.level = level * drv_data->scale;
+	g_debug ("Light read from IIO: %lf, (scale %lf)", level, drv_data->scale);
 
 	/* Even though the IIO kernel API declares in_intensity* values as unitless,
 	 * we use Microsoft's hid-sensors-usages.docx which mentions that Windows 8
@@ -69,23 +66,45 @@ light_changed (gpointer user_data)
 	return G_SOURCE_CONTINUE;
 }
 
+static char *
+get_illuminance_channel_path (GUdevDevice *device,
+			      const char *suffix)
+{
+	const char *channels[] = {
+		"in_illuminance",
+		"in_illuminance0"
+	};
+	char *path = NULL;
+	guint i;
+
+	for (i = 0; i < G_N_ELEMENTS (channels); i++) {
+		path = g_strdup_printf ("%s/%s_%s",
+					g_udev_device_get_sysfs_path (device),
+					channels[i],
+					suffix);
+		if (g_file_test (path, G_FILE_TEST_EXISTS))
+			return path;
+		g_clear_pointer (&path, g_free);
+	}
+	return NULL;
+}
+
 static guint
 get_interval (GUdevDevice *device)
 {
-	gdouble time;
+	gdouble time = DEFAULT_POLL_TIME;
 	char *path, *contents;
 
-	path = g_build_filename (g_udev_device_get_sysfs_path (device),
-				 "in_illuminance_integration_time",
-				 NULL);
+	path = get_illuminance_channel_path (device, "integration_time");
+	if (!path)
+		goto out;
 	if (g_file_get_contents (path, &contents, NULL, NULL)) {
 		time = g_ascii_strtod (contents, NULL);
 		g_free (contents);
-	} else {
-		time = DEFAULT_POLL_TIME;
 	}
 	g_free (path);
 
+out:
 	return (time * 1000);
 }
 
@@ -122,9 +141,21 @@ iio_poll_light_open (GUdevDevice        *device,
 	drv_data->user_data = user_data;
 
 	drv_data->interval = get_interval (device);
-	drv_data->input_path = g_build_filename (g_udev_device_get_sysfs_path (device),
-						 "in_illuminance_input",
-						 NULL);
+	drv_data->input_path = get_illuminance_channel_path (device, "input");
+	if (!drv_data->input_path)
+		drv_data->input_path = get_illuminance_channel_path (device, "raw");
+	if (!drv_data->input_path)
+		return FALSE;
+
+	if (g_str_has_prefix (drv_data->input_path, "in_illuminance0")) {
+		drv_data->scale = g_udev_device_get_sysfs_attr_as_double (device,
+									  "in_illuminance0_scale");
+	} else {
+		drv_data->scale = g_udev_device_get_sysfs_attr_as_double (device,
+									  "in_illuminance_scale");
+	}
+	if (drv_data->scale == 0.0)
+		drv_data->scale = 1.0;
 
 	return TRUE;
 }
